// @flow

import React from 'react'
import cn from 'classnames'
import { themr } from 'react-css-themr'
import Icon from 'components/Icon'

export type TContact = {
  address: string | string[],
  phone: string | string[],
  email: string | string[],
}

const Item = ({
  theme,
  icon,
  label,
  values,
}: {
  theme: TTheme,
  icon?: ?string,
  label?: ?string,
  values: string[],
}) => {
  if (!values || values.filter(x => x).length === 0) return null
  return (
    <div className={theme.itemWrapper}>
      <div className={theme.labelWrapper}>
        {icon && <Icon icon={icon} className={theme.icon} />}
        {label && <span className={theme.label}>{label}</span>}
      </div>
      <div className={theme.valueWrapper}>
        {values.filter(x => x).map((value, idx) => (
          <div key={idx} className={theme.value}>
            {value}
          </div>
        ))}
      </div>
    </div>
  )
}

function ContactInfo(props: {
  className?: string,
  contact: TContact,
  icons?: { [key: string]: string },
  labels?: { [key: string]: string },
  showLabel?: boolean,
  showIcon?: boolean,
  theme?: TTheme,
}) {
  const {
    className,
    contact: { address: _address, phone: _phone, email: _email } = {},
    icons = {},
    labels = {},
    showLabel = true,
    showIcon = true,
    theme = {},
    ...rest
  } = props
  if (!_address && !_phone && !_email) return null
  const address: string[] = Array.isArray(_address) ? _address : [_address]
  const phone: string[] = Array.isArray(_phone) ? _phone : [_phone]
  const email: string[] = Array.isArray(_email) ? _email : [_email]
  const list = { address, phone, email }
  const keys = Object.keys(list)
  return (
    <div
      {...rest}
      data-component="ContactInfo"
      className={cn(theme.root, className)}
    >
      {keys.map((key) => {
        const icon = showIcon ? icons[key] : null
        const label = showLabel ? labels[key] : null
        const values = list[key]
        return (
          <Item
            key={key}
            icon={icon}
            label={label}
            values={values}
            theme={theme}
          />
        )
      })}
    </div>
  )
}

const defaultStyle = {
  root: '',
  itemWrapper: '',
  labelWrapper: 'pa1 bb b--light-gray',
  icon: 'mr2',
  label: '',
  valueWrapper: 'pa2',
  value: '',

}

export default themr('ContactInfo', defaultStyle, { composeTheme: 'softly' })(ContactInfo)
