// @flow

import React from 'react'
import { pipe, filter, reverse, uniqBy, map } from 'ramda'

export function getDefaultMetaTags({ favicons }: { favicons: { sizes: string, href: string }[] }) {
  return [
    { tag: 'meta', name: 'robots', content: 'index,follow' },
    { tag: 'meta', name: 'googlebot', content: 'index,follow' },
  ].concat(
    (favicons || []).map(({ sizes, href }) => ({
      tag: 'link',
      rel: 'icon',
      type: 'image/png',
      sizes,
      href,
    })),
  )
}

export function getGeneralHead({
  url,
  title,
  description,
  subject,
}: {
  appId?: string,
  url?: string,
  title?: string,
  description?: string,
  subject?: string,
}) {
  return [
    title && { tag: 'title', children: title },
    url && { tag: 'link', rel: 'canonical', href: url },
    description && { tag: 'meta', name: 'description', content: description },
    { tag: 'meta', name: 'subject', content: subject },
  ]
}

function getFacebookHead({
  appId,
  url,
  title,
  description,
  imageUrl,
  siteName,
  locale,
  author,
}: {
  appId?: string,
  url?: string,
  title?: string,
  description?: string,
  imageUrl?: string,
  siteName?: string,
  locale?: string,
  author?: string,
}) {
  return [
    appId && { tag: 'meta', property: 'fb:app_id', content: appId },
    url && { tag: 'meta', property: 'og:url', content: url },
    { tag: 'meta', property: 'og:type', content: 'website' },
    title && { tag: 'meta', property: 'og:title', content: title },
    imageUrl && { tag: 'meta', property: 'og:image', content: imageUrl },
    description && {
      tag: 'meta',
      property: 'og:description',
      content: description,
    },
    siteName && { tag: 'meta', property: 'og:site_name', content: siteName },
    locale && { tag: 'meta', property: 'og:locale', content: locale },
    author && { tag: 'meta', property: 'article:author', content: author },
  ]
}

function geTwitterHead({
  url,
  title,
  description,
  imageUrl,
  twSiteAccount,
  twCreator,
}: {
  url?: string,
  title?: string,
  description?: string,
  imageUrl?: string,
  twSiteAccount?: string,
  twCreator?: string,
}) {
  return [
    { tag: 'meta', name: 'twitter:card', content: 'summary' },
    twSiteAccount && {
      tag: 'meta',
      name: 'twitter:site',
      content: twSiteAccount,
    },
    twCreator && { tag: 'meta', name: 'twitter:creator', content: twCreator },
    url && { tag: 'meta', name: 'twitter:url', content: url },
    title && { tag: 'meta', name: 'twitter:title', content: title },
    description && {
      tag: 'meta',
      name: 'twitter:description',
      content: description,
    },
    imageUrl && { tag: 'meta', name: 'twitter:image', content: imageUrl },
  ]
}

function getGooglePlusHead({
  title,
  description,
  imageUrl,
  googlePlusPublisher,
}: {
  title?: string,
  description?: string,
  imageUrl?: string,
  googlePlusPublisher?: string,
}) {
  return [
    googlePlusPublisher && {
      tag: 'link',
      href: googlePlusPublisher,
      rel: 'publisher',
    },
    title && { tag: 'meta', itemProp: 'name', content: title },
    description && {
      tag: 'meta',
      itemProp: 'description',
      content: description,
    },
    imageUrl && { tag: 'meta', itemProp: 'image', content: imageUrl },
  ]
}

const generateKey = ({ tag, rel, property, name, itemProp }) =>
  `${tag}:${rel}:${property}:${name}:${itemProp}`

const transform = pipe(
  filter(x => x),
  reverse,
  uniqBy(generateKey),
  map((item) => {
    const { tag: Tag = 'meta', ...rest } = item
    return <Tag key={generateKey(item)} {...rest} />
  }),
)

export default function getMetaTags({
  url,
  title,
  description,
  subject,
  favicons,
  imageUrl,
  siteName,
  locale,
  author,
  fbAppId,
  twSiteAccount,
  twCreator,
  googlePlusPublisher,
  meta = [],
}: TMeta) {
  const tagList = [
    ...getGeneralHead({
      url,
      title,
      description,
      subject,
    }),
    ...getDefaultMetaTags({
      favicons,
    }),
    ...getFacebookHead({
      fbAppId,
      author,
      url,
      title,
      description,
      imageUrl,
      siteName,
      locale,
    }),
    ...geTwitterHead({
      url,
      title,
      description,
      imageUrl,
      twSiteAccount,
      twCreator,
    }),
    ...getGooglePlusHead({
      title,
      description,
      imageUrl,
      googlePlusPublisher,
    }),
    ...meta,
  ]
  return transform(tagList)
}
